@@ -77,24 +77,27 @@
 //!
 //! # APIs
 //! Layouts are defined using the [define_layout!] macro. Based on such a layout, this library offers two alternative APIs for data access:
-//! 1. The [Field] API that offers free functions to read/write the data based on an underlying slice of storage (`packet_data` in the example above) holding the packet data. This API does not wrap the underlying slice of storage data, which means you have to pass it in to each accessor.
-//!    This is not the API used in the example above, see [Field] for an API example.
-//! 2. The [FieldView] API that wraps a slice of storage data and remembers it in a `View` object, allowing access to the fields without having to pass in the packed data slice each time. This is the API used in the example above. See [FieldView] for another example.
+//! 1. The [trait@Field] API that offers free functions to read/write the data based on an underlying slice of storage (`packet_data` in the example above) holding the packet data. This API does not wrap the underlying slice of storage data, which means you have to pass it in to each accessor.
+//!    This is not the API used in the example above, see [trait@Field] for an API example.
+//! 2. The [struct@FieldView] API that wraps a slice of storage data and remembers it in a `View` object, allowing access to the fields without having to pass in the packed data slice each time. This is the API used in the example above. See [trait@FieldView] for another example.
 //!
 //! ## Supported field types
 //! ### Primitive integer types
 //! - [u8](https://doc.rust-lang.org/stable/std/primitive.u8.html), [u16](https://doc.rust-lang.org/stable/std/primitive.u16.html), [u32](https://doc.rust-lang.org/stable/std/primitive.u32.html), [u64](https://doc.rust-lang.org/stable/std/primitive.u64.html)
 //! - [i8](https://doc.rust-lang.org/stable/std/primitive.i8.html), [i16](https://doc.rust-lang.org/stable/std/primitive.i16.html), [i32](https://doc.rust-lang.org/stable/std/primitive.i32.html), [i64](https://doc.rust-lang.org/stable/std/primitive.i64.html)
 //!
-//! For these fields, the [Field] API offers [Field::read], [Field::write] and the [FieldView] API offers [FieldView::read] and [FieldView::write].
+//! For these fields, the [trait@Field] API offers [Field::read], [Field::write] and the [struct@FieldView] API offers [FieldView::read] and [FieldView::write].
 //!
 //! ### Fixed size byte arrays: `[u8; N]`.
-//! For these fields, the [Field] API offers [Field::data], [Field::data_mut], and the [FieldView] API offers [FieldView::data] and [FieldView::data_mut].
+//! For these fields, the [trait@Field] API offers [Field::data], [Field::data_mut], and the [struct@FieldView] API offers [FieldView::data] and [FieldView::data_mut].
 //!
 //! ### Open ended byte arrays: `[u8]`.
 //! This field type can only occur as the last field of a layout and will mach the remaining data until the end of the storage.
 //! This field has a dynamic size, depending on how large the package data is.
-//! For these fields, the [Field] API offers [Field::data], [Field::data_mut](Field::data_mut) and the [FieldView] API offers [FieldView::data], [FieldView::data_mut] and [FieldView::extract].
+//! For these fields, the [trait@Field] API offers [Field::data], [Field::data_mut] and the [struct@FieldView] API offers [FieldView::data], [FieldView::data_mut] and [FieldView::extract].
+//!
+//! ### Custom field types
+//! You can define your own custom types as long as they implement the [trait@LayoutAs] trait to define how to convert them from/to a primitive type.
 //!
 //! # Data types maybe supported in the future
 //! These data types aren't supported yet, but they could be added in theory and might be added in future versions.
@@ -111,7 +114,7 @@
 //! Both of these, however, would be some effort to implement and it is unclear if that will ever happen (unless somebody opens a PR for it).
 //!
 //! ### Strings
-//! For strings, note that even fixed-size UTF-8 strings take a variable number of characters because of the UTF-8 encoding and that brings all the issues of data types with dynamic length with it.
+//! For strings, note that even fixed-size UTF-8 strings take a variable number of bytes because of the UTF-8 encoding and that brings all the issues of data types with dynamic length with it.
 //! This is why strings aren't supported yet.
 //!
 //! ### Fixed-size arrays other than `[u8; N]`
@@ -122,13 +125,20 @@
 #![forbid(unsafe_code)]
 #![deny(missing_docs)]
 
-pub mod example;
+mod endianness;
 mod fields;
 mod macro_define_layout;
+mod view;
 
+pub mod example;
+
+pub use endianness::{BigEndian, LittleEndian};
 pub use fields::{
-    BigEndian, Field, FieldMetadata, FieldSize, FieldView, LittleEndian, SizedFieldMetadata,
+    primitive::PrimitiveField,
+    wrapped::{LayoutAs, WrappedField},
+    Field, FieldCopyAccess, FieldSliceAccess, SizedField,
 };
+pub use view::FieldView;
 
 pub use doc_comment::doc_comment;
 pub use paste::paste;
@@ -140,6 +150,6 @@ pub use paste::paste;
 /// use binary_layout::prelude::*;
 /// ```
 pub mod prelude {
-    pub use super::{FieldMetadata, SizedFieldMetadata};
+    pub use super::{Field, FieldCopyAccess, FieldSliceAccess, SizedField};
     pub use crate::define_layout;
 }
