@@ -42,6 +42,9 @@
 ///   - data accessors for the [Field](crate::Field) API
 /// - The module will also contain a `View` struct that offers the [FieldView](crate::FieldView) API.
 ///
+/// This macro will also generate rustdoc documentation for everything it generates. One of the best ways to figure out
+/// how to use the generated layouts is to read the rustdoc documentation that was generated for them.
+///
 /// ## Metadata Example
 /// ```
 /// use binary_layout::prelude::*;
@@ -72,58 +75,88 @@
 /// - `into_${field_name}`: Extract access. This destroys the `View` and returns a [FieldView](crate::FieldView) instance owning the storage. Mostly useful for [FieldView::extract](crate::FieldView::extract).
 #[macro_export]
 macro_rules! define_layout {
-    ($name: ident, $endianness: ident, {$($field_name: ident : $field_type: ty),* $(,)?}) => {
-        /// This module is autogenerated. It defines a layout using the [binary_layout](crate) crate.
-        #[allow(dead_code)]
-        pub mod $name {
-            #[allow(unused_imports)]
-            use super::*;
+    ($name: ident, $endianness: ident, {$($field_name: ident : $field_type: ty $(as $underlying_type: ty)?),* $(,)?}) => {
+        $crate::doc_comment!{
+            concat!{"
+            This module is autogenerated. It defines a layout using the [binary_layout](crate) crate based on the following definition:
+            ```ignore
+            define_layout!(", stringify!($name), ", ", stringify!($endianness), ", {", $("
+                ", stringify!($field_name), ": ", stringify!($field_type), $(" as ", stringify!($underlying_type), )? ",", )* "
+            });
+            ```
+            "},
+            #[allow(dead_code)]
+            pub mod $name {
+                #[allow(unused_imports)]
+                use super::*;
 
-            $crate::define_layout!(_impl_fields $crate::$endianness, 0, {$($field_name : $field_type),*});
+                $crate::define_layout!(_impl_fields $crate::$endianness, 0, {$($field_name : $field_type $(as $underlying_type)?),*});
 
-            /// The [View] struct defines the [FieldView](crate::FieldView) API.
-            /// An instance of [View] wraps a storage (either borrowed or owned)
-            /// and allows accessors for the layout fields.
-            pub struct View<S> {
-                storage: S,
-            }
-            impl <S> View<S> {
-                /// You can create views over a storage by calling [View::new].
-                ///
-                /// `S` is the type of underlying storage. It can be
-                /// - Immutable borrowed storage: `&[u8]`
-                /// - Mutable borrowed storage: `&mut [u8]`
-                /// - Owning storage: impl `AsRef<u8>` (for example: `Vec<u8>`)
-                pub fn new(storage: S) -> Self {
-                    Self {storage}
+                $crate::doc_comment!{
+                    concat!{"
+                    The [View] struct defines the [FieldView](crate::FieldView) API.
+                    An instance of [View] wraps a storage (either borrowed or owned)
+                    and allows accessors for the layout fields.
+
+                    This view is based on the following layout definition:
+                    ```ignore
+                    define_layout!(", stringify!($name), ", ", stringify!($endianness), ", {", $("
+                        ", stringify!($field_name), ": ", stringify!($field_type), $(" as ", stringify!($underlying_type), )? ",",)* "
+                    });
+                    ```
+                    "},
+                    pub struct View<S> {
+                        storage: S,
+                    }
                 }
+                impl <S> View<S> {
+                    /// You can create views over a storage by calling [View::new].
+                    ///
+                    /// `S` is the type of underlying storage. It can be
+                    /// - Immutable borrowed storage: `&[u8]`
+                    /// - Mutable borrowed storage: `&mut [u8]`
+                    /// - Owning storage: impl `AsRef<u8>` (for example: `Vec<u8>`)
+                    pub fn new(storage: S) -> Self {
+                        Self {storage}
+                    }
 
-                /// This destroys the view and returns the underlying storage back to you.
-                /// This is useful if you created an owning view (e.g. vased on `Vec<u8>`)
-                /// and now need the underlying `Vec<u8>` back.
-                pub fn into_storage(self) -> S {
-                    self.storage
+                    /// This destroys the view and returns the underlying storage back to you.
+                    /// This is useful if you created an owning view (e.g. vased on `Vec<u8>`)
+                    /// and now need the underlying `Vec<u8>` back.
+                    pub fn into_storage(self) -> S {
+                        self.storage
+                    }
+
+                    $crate::define_layout!(_impl_view_into {$($field_name),*});
+                }
+                impl <S: AsRef<[u8]>> View<S> {
+                    $crate::define_layout!(_impl_view_asref {$($field_name),*});
+                }
+                impl <S: AsMut<[u8]>> View<S> {
+                    $crate::define_layout!(_impl_view_asmut {$($field_name),*});
                 }
-
-                $crate::define_layout!(_impl_view_into {$($field_name),*});
-            }
-            impl <S: AsRef<[u8]>> View<S> {
-                $crate::define_layout!(_impl_view_asref {$($field_name),*});
-            }
-            impl <S: AsMut<[u8]>> View<S> {
-                $crate::define_layout!(_impl_view_asmut {$($field_name),*});
             }
         }
     };
 
     (_impl_fields $endianness: ty, $offset_accumulator: expr, {}) => {};
-    (_impl_fields $endianness: ty, $offset_accumulator: expr, {$name: ident : $type: ty $(, $name_tail: ident : $type_tail: ty)*}) => {
+    (_impl_fields $endianness: ty, $offset_accumulator: expr, {$name: ident : $type: ty as $underlying_type: ty $(, $name_tail: ident : $type_tail: ty $(as $underlying_tail: ty)?)*}) => {
         $crate::doc_comment!{
             concat!("Metadata and [Field](binary_layout::Field) API accessors for the `", stringify!($name), "` field"),
             #[allow(non_camel_case_types)]
-            pub type $name = $crate::Field::<$type, $endianness, $offset_accumulator>;
+            pub type $name = $crate::WrappedField::<$underlying_type, $type, $crate::PrimitiveField::<$underlying_type, $endianness, $offset_accumulator>>;
         }
-        $crate::define_layout!(_impl_fields $endianness, {($offset_accumulator + <$type as $crate::FieldSize>::SIZE)}, {$($name_tail : $type_tail),*});
+        $crate::define_layout!(_impl_fields $endianness, {($offset_accumulator + <$name as $crate::SizedField>::SIZE)}, {$($name_tail : $type_tail $(as $underlying_tail)?),*});
+    };
+
+    (_impl_fields $endianness: ty, $offset_accumulator: expr, {}) => {};
+    (_impl_fields $endianness: ty, $offset_accumulator: expr, {$name: ident : $type: ty $(, $name_tail: ident : $type_tail: ty $(as $underlying_tail: ty)?)*}) => {
+        $crate::doc_comment!{
+            concat!("Metadata and [Field](binary_layout::Field) API accessors for the `", stringify!($name), "` field"),
+            #[allow(non_camel_case_types)]
+            pub type $name = $crate::PrimitiveField::<$type, $endianness, $offset_accumulator>;
+        }
+        $crate::define_layout!(_impl_fields $endianness, {($offset_accumulator + <$name as $crate::SizedField>::SIZE)}, {$($name_tail : $type_tail $(as $underlying_tail)?),*});
     };
 
     (_impl_view_asref {}) => {};
@@ -166,7 +199,7 @@ macro_rules! define_layout {
 
 #[cfg(test)]
 mod tests {
-    use crate::{FieldMetadata, SizedFieldMetadata};
+    use crate::{Field, FieldCopyAccess, FieldSliceAccess, SizedField};
 
     use rand::{rngs::StdRng, RngCore, SeedableRng};
     use std::convert::TryInto;
@@ -673,6 +706,233 @@ mod tests {
         }
     }
 
+    mod wrapped {
+        use super::*;
+        use crate::LayoutAs;
+
+        #[derive(Debug, PartialEq, Eq)]
+        pub struct Wrapped<T>(T);
+        impl<T> LayoutAs<T> for Wrapped<T> {
+            fn read(v: T) -> Wrapped<T> {
+                Wrapped(v)
+            }
+
+            fn write(v: Wrapped<T>) -> T {
+                v.0
+            }
+        }
+
+        define_layout!(noslice, LittleEndian, {
+            first: Wrapped<i8> as i8,
+            second: Wrapped<i64> as i64,
+            third: Wrapped<u16> as u16,
+        });
+        #[test]
+        fn metadata() {
+            assert_eq!(0, noslice::first::OFFSET);
+            assert_eq!(1, noslice::first::SIZE);
+            assert_eq!(1, noslice::second::OFFSET);
+            assert_eq!(8, noslice::second::SIZE);
+            assert_eq!(9, noslice::third::OFFSET);
+            assert_eq!(2, noslice::third::SIZE);
+        }
+
+        #[test]
+        fn fields() {
+            let mut storage = data_region(1024, 5);
+
+            // Test initial data is read correctly
+            assert_eq!(
+                Wrapped(i8::from_le_bytes(
+                    (&data_region(1024, 5)[0..1]).try_into().unwrap()
+                )),
+                noslice::first::read(&storage)
+            );
+            assert_eq!(
+                Wrapped(i64::from_le_bytes(
+                    (&data_region(1024, 5)[1..9]).try_into().unwrap()
+                )),
+                noslice::second::read(&storage)
+            );
+            assert_eq!(
+                Wrapped(u16::from_le_bytes(
+                    (&data_region(1024, 5)[9..11]).try_into().unwrap()
+                )),
+                noslice::third::read(&storage)
+            );
+
+            // Test data can be written
+            noslice::first::write(&mut storage, Wrapped(60));
+            noslice::second::write(&mut storage, Wrapped(-100_000_000_000));
+            noslice::third::write(&mut storage, Wrapped(1_000));
+
+            // Test reading will return changed data
+            assert_eq!(Wrapped(60), noslice::first::read(&storage));
+            assert_eq!(Wrapped(-100_000_000_000), noslice::second::read(&storage));
+            assert_eq!(Wrapped(1_000), noslice::third::read(&storage));
+        }
+
+        #[test]
+        fn view_readonly() {
+            let storage = data_region(1024, 5);
+            let view = noslice::View::new(&storage);
+
+            // Test initial data is read correctly
+            assert_eq!(
+                Wrapped(i8::from_le_bytes(
+                    (&data_region(1024, 5)[0..1]).try_into().unwrap()
+                )),
+                view.first().read()
+            );
+            assert_eq!(
+                Wrapped(i64::from_le_bytes(
+                    (&data_region(1024, 5)[1..9]).try_into().unwrap()
+                )),
+                view.second().read()
+            );
+            assert_eq!(
+                Wrapped(u16::from_le_bytes(
+                    (&data_region(1024, 5)[9..11]).try_into().unwrap()
+                )),
+                view.third().read()
+            );
+
+            // Test into_storage will return correct data
+            let extracted_storage = view.into_storage();
+            assert_eq!(extracted_storage, &storage);
+        }
+
+        #[test]
+        fn view_readwrite() {
+            let mut storage = data_region(1024, 5);
+            let mut view = noslice::View::new(&mut storage);
+
+            // Test initial data is read correctly
+            assert_eq!(
+                Wrapped(i8::from_le_bytes(
+                    (&data_region(1024, 5)[0..1]).try_into().unwrap()
+                )),
+                view.first().read()
+            );
+            assert_eq!(
+                Wrapped(i64::from_le_bytes(
+                    (&data_region(1024, 5)[1..9]).try_into().unwrap()
+                )),
+                view.second().read()
+            );
+            assert_eq!(
+                Wrapped(u16::from_le_bytes(
+                    (&data_region(1024, 5)[9..11]).try_into().unwrap()
+                )),
+                view.third().read()
+            );
+
+            // Test data can be written
+            view.first_mut().write(Wrapped(50));
+            view.second_mut().write(Wrapped(10i64.pow(15)));
+            view.third_mut().write(Wrapped(1000));
+
+            // Test reading will return changed data
+            assert_eq!(Wrapped(50), view.first().read());
+            assert_eq!(Wrapped(10i64.pow(15)), view.second().read());
+            assert_eq!(Wrapped(1000), view.third().read());
+
+            // Test into_storage will return correct data
+            let extracted_storage = view.into_storage().clone();
+            assert_eq!(&storage, &extracted_storage);
+
+            // Test original storage is actually changed
+            assert_eq!(50, i8::from_le_bytes((&storage[0..1]).try_into().unwrap()));
+            assert_eq!(
+                10i64.pow(15),
+                i64::from_le_bytes((&storage[1..9]).try_into().unwrap())
+            );
+            assert_eq!(
+                1000,
+                u16::from_le_bytes((&storage[9..11]).try_into().unwrap())
+            );
+        }
+
+        #[test]
+        fn view_vec_readonly() {
+            let view = noslice::View::new(data_region(1024, 5));
+
+            // Test initial data is read correctly
+            assert_eq!(
+                Wrapped(i8::from_le_bytes(
+                    (&data_region(1024, 5)[0..1]).try_into().unwrap()
+                )),
+                view.first().read()
+            );
+            assert_eq!(
+                Wrapped(i64::from_le_bytes(
+                    (&data_region(1024, 5)[1..9]).try_into().unwrap()
+                )),
+                view.second().read()
+            );
+            assert_eq!(
+                Wrapped(u16::from_le_bytes(
+                    (&data_region(1024, 5)[9..11]).try_into().unwrap()
+                )),
+                view.third().read()
+            );
+
+            // Test into_storage will return correct data
+            let extracted_storage = view.into_storage();
+            assert_eq!(&data_region(1024, 5), &extracted_storage);
+        }
+
+        #[test]
+        fn view_vec_readwrite() {
+            let mut view = noslice::View::new(data_region(1024, 5));
+
+            // Test initial data is read correctly
+            assert_eq!(
+                Wrapped(i8::from_le_bytes(
+                    (&data_region(1024, 5)[0..1]).try_into().unwrap()
+                )),
+                view.first().read()
+            );
+            assert_eq!(
+                Wrapped(i64::from_le_bytes(
+                    (&data_region(1024, 5)[1..9]).try_into().unwrap()
+                )),
+                view.second().read()
+            );
+            assert_eq!(
+                Wrapped(u16::from_le_bytes(
+                    (&data_region(1024, 5)[9..11]).try_into().unwrap()
+                )),
+                view.third().read()
+            );
+
+            // Test data can be written
+            view.first_mut().write(Wrapped(50));
+            view.second_mut().write(Wrapped(10i64.pow(15)));
+            view.third_mut().write(Wrapped(1000));
+
+            // Test reading will return changed data
+            assert_eq!(Wrapped(50), view.first().read());
+            assert_eq!(Wrapped(10i64.pow(15)), view.second().read());
+            assert_eq!(Wrapped(1000), view.third().read());
+
+            // Test into_storage will return correct data
+            let extracted_storage = view.into_storage();
+            assert_eq!(
+                50,
+                i8::from_le_bytes((&extracted_storage[0..1]).try_into().unwrap())
+            );
+            assert_eq!(
+                10i64.pow(15),
+                i64::from_le_bytes((&extracted_storage[1..9]).try_into().unwrap())
+            );
+            assert_eq!(
+                1000,
+                u16::from_le_bytes((&extracted_storage[9..11]).try_into().unwrap())
+            );
+        }
+    }
+
     #[test]
     fn can_be_created_with_and_without_trailing_comma() {
         define_layout!(first, LittleEndian, { field: u8 });
@@ -765,7 +1025,7 @@ mod tests {
         let mut storage = data_region(1024, 0);
         let extracted: &mut [u8] = {
             let view: layout::View<&mut [u8]> = layout::View::new(&mut storage);
-            view.into_tail().extract()
+            view.into_tail().extract_mut()
         };
 
         assert_eq!(&data_region(1024, 0)[1..], extracted);
@@ -781,7 +1041,7 @@ mod tests {
         let mut storage = data_region(1024, 0);
         let extracted: &mut [u8] = {
             let view: layout::View<&mut Vec<u8>> = layout::View::new(&mut storage);
-            view.into_tail().extract()
+            view.into_tail().extract_mut()
         };
 
         assert_eq!(&data_region(1024, 0)[1..], extracted);
